@@ -23,21 +23,22 @@ import warnings
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import excutils
-from oslo_utils import imageutils
-from oslo_utils import units
-import tenacity
 
 from ironic_lib.common.i18n import _
 from ironic_lib import disk_partitioner
 from ironic_lib import exception
+from ironic_lib import qemu_img
 from ironic_lib import utils
 
 
 opts = [
     cfg.IntOpt('efi_system_partition_size',
-               default=200,
+               default=550,
                help='Size of EFI system partition in MiB when configuring '
-                    'UEFI systems for local boot.'),
+                    'UEFI systems for local boot. A common minimum is ~200 '
+                    'megabytes, however OS driven firmware updates and '
+                    'unikernel usage generally requires more space on the '
+                    'efi partition.'),
     cfg.IntOpt('bios_boot_partition_size',
                default=1,
                help='Size of BIOS Boot partition in MiB when configuring '
@@ -53,13 +54,6 @@ opts = [
                default=10,
                help='Maximum number of attempts to try to read the '
                     'partition.'),
-    cfg.IntOpt('image_convert_memory_limit',
-               default=2048,
-               help='Memory limit for "qemu-img convert" in MiB. Implemented '
-                    'via the address space resource limit.'),
-    cfg.IntOpt('image_convert_attempts',
-               default=3,
-               help='Number of attempts to convert an image.'),
 ]
 
 CONF = cfg.CONF
@@ -69,6 +63,8 @@ LOG = logging.getLogger(__name__)
 
 _PARTED_PRINT_RE = re.compile(r"^(\d+):([\d\.]+)MiB:"
                               r"([\d\.]+)MiB:([\d\.]+)MiB:(\w*):(.*):(.*);")
+_PARTED_TABLE_TYPE_RE = re.compile(r'^.*partition\s+table\s*:\s*(gpt|msdos)',
+                                   re.IGNORECASE | re.MULTILINE)
 
 CONFIGDRIVE_LABEL = "config-2"
 MAX_CONFIG_DRIVE_SIZE_MB = 64
@@ -78,17 +74,9 @@ GPT_SIZE_SECTORS = 33
 # Maximum disk size supported by MBR is 2TB (2 * 1024 * 1024 MB)
 MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR = 2097152
 
-# Limit the memory address space to 1 GiB when running qemu-img
-QEMU_IMG_LIMITS = None
-
-
-def _qemu_img_limits():
-    global QEMU_IMG_LIMITS
-    if QEMU_IMG_LIMITS is None:
-        QEMU_IMG_LIMITS = processutils.ProcessLimits(
-            address_space=CONF.disk_utils.image_convert_memory_limit
-            * units.Mi)
-    return QEMU_IMG_LIMITS
+# Backward compatibility, do not use
+qemu_img_info = qemu_img.image_info
+convert_image = qemu_img.convert_image
 
 
 def list_partitions(device):
@@ -147,7 +135,7 @@ def count_mbr_partitions(device):
     output = output.replace('<', '').replace('>', '')
     partitions = [int(s) for s in output.split() if s.isdigit()]
 
-    return(sum(i < 5 for i in partitions), sum(i > 4 for i in partitions))
+    return (sum(i < 5 for i in partitions), sum(i > 4 for i in partitions))
 
 
 def get_disk_identifier(dev):
@@ -176,7 +164,43 @@ def get_partition_table_type(device):
     :param device: the name of the device
     :return: dos, gpt or None
     """
-    return get_device_information(device, probe=True).get('PTTYPE')
+    out = utils.execute('parted', '--script', device, '--', 'print',
+                        run_as_root=True, use_standard_locale=True)[0]
+    m = _PARTED_TABLE_TYPE_RE.search(out)
+    if m:
+        return m.group(1)
+
+    LOG.warning("Unable to get partition table type for device %s", device)
+    return 'unknown'
+
+
+def _blkid(device, probe=False, fields=None):
+    args = []
+    if probe:
+        args.append('-p')
+    if fields:
+        args += sum((['-s', field] for field in fields), [])
+
+    output, err = utils.execute('blkid', device, *args,
+                                use_standard_locale=True, run_as_root=True)
+    if output.strip():
+        return output.split(': ', 1)[1]
+    else:
+        return ""
+
+
+def _lsblk(device, deps=True, fields=None):
+    args = ['--pairs', '--bytes', '--ascii']
+    if not deps:
+        args.append('--nodeps')
+    if fields:
+        args.extend(['--output', ','.join(fields)])
+    else:
+        args.append('--output-all')
+
+    output, err = utils.execute('lsblk', device, *args,
+                                use_standard_locale=True, run_as_root=True)
+    return output.strip()
 
 
 def get_device_information(device, probe=False, fields=None):
@@ -185,21 +209,17 @@ def get_device_information(device, probe=False, fields=None):
     Can be applied to all block devices: disks, RAID, partitions.
 
     :param device: Device name.
-    :param probe: Switch to low-level probing mode.
+    :param probe: DEPRECATED, do not use.
     :param fields: A list of fields to request (all by default).
     :return: A dictionary with requested fields as keys.
     :raises: ProcessExecutionError
     """
-    args = []
     if probe:
-        args.append('--probe')
-    if fields:
-        args += sum((['--match-tag', field] for field in fields), [])
+        output = _blkid(device, probe=True, fields=fields)
+    else:
+        output = _lsblk(device, fields=fields, deps=False)
 
-    output, err = utils.execute('blkid', device, *args,
-                                use_standard_locale=True, run_as_root=True)
-    if output.strip():
-        output = output.split(': ', 1)[1]
+    if output:
         return next(utils.parse_device_tags(output))
     else:
         return {}
@@ -248,8 +268,7 @@ def get_uefi_disk_identifier(dev):
             vals = line.split()
             partition_id = vals[0]
     try:
-        lsblk_output, _ = utils.execute('lsblk', '-PbioUUID', partition_id,
-                                        run_as_root=True)
+        lsblk_output = _lsblk(partition_id, fields=['UUID'])
         disk_identifier = lsblk_output.split("=")[1].strip()
         disk_identifier = disk_identifier.strip('"')
     except processutils.ProcessExecutionError as e:
@@ -449,71 +468,12 @@ def dd(src, dst, conv_flags=None):
              *extra_args)
 
 
-def qemu_img_info(path):
-    """Return an object containing the parsed output from qemu-img info."""
-    if not os.path.exists(path):
-        return imageutils.QemuImgInfo()
-
-    out, err = utils.execute('env', 'LC_ALL=C', 'LANG=C',
-                             'qemu-img', 'info', path,
-                             '--output=json',
-                             prlimit=_qemu_img_limits())
-    return imageutils.QemuImgInfo(out, format='json')
-
-
-def _retry_on_res_temp_unavailable(exc):
-    if (isinstance(exc, processutils.ProcessExecutionError)
-            and ('Resource temporarily unavailable' in exc.stderr
-                 or 'Cannot allocate memory' in exc.stderr)):
-        return True
-    return False
-
-
-@tenacity.retry(
-    retry=tenacity.retry_if_exception(_retry_on_res_temp_unavailable),
-    stop=tenacity.stop_after_attempt(CONF.disk_utils.image_convert_attempts),
-    reraise=True)
-def convert_image(source, dest, out_format, run_as_root=False, cache=None,
-                  out_of_order=False):
-    """Convert image to other format."""
-    cmd = ['qemu-img', 'convert', '-O', out_format]
-    if cache is not None:
-        cmd += ['-t', cache]
-    if out_of_order:
-        cmd.append('-W')
-    cmd += [source, dest]
-    # NOTE(TheJulia): Staticly set the MALLOC_ARENA_MAX to prevent leaking
-    # and the creation of new malloc arenas which will consume the system
-    # memory. If limited to 1, qemu-img consumes ~250 MB of RAM, but when
-    # another thread tries to access a locked section of memory in use with
-    # another thread, then by default a new malloc arena is created,
-    # which essentially balloons the memory requirement of the machine.
-    # Default for qemu-img is 8 * nCPU * ~250MB (based on defaults +
-    # thread/code/process/library overhead. In other words, 64 GB. Limiting
-    # this to 3 keeps the memory utilization in happy cases below the overall
-    # threshold which is in place in case a malicious image is attempted to
-    # be passed through qemu-img.
-    env_vars = {'MALLOC_ARENA_MAX': '3'}
-    try:
-        utils.execute(*cmd, run_as_root=run_as_root,
-                      prlimit=_qemu_img_limits(),
-                      use_standard_locale=True,
-                      env_variables=env_vars)
-    except processutils.ProcessExecutionError as e:
-        if ('Resource temporarily unavailable' in e.stderr
-            or 'Cannot allocate memory' in e.stderr):
-            LOG.debug('Failed to convert image, retrying. Error: %s', e)
-            # Sync disk caches before the next attempt
-            utils.execute('sync')
-        raise
-
-
 def populate_image(src, dst, conv_flags=None):
-    data = qemu_img_info(src)
+    data = qemu_img.image_info(src)
     if data.file_format == 'raw':
         dd(src, dst, conv_flags=conv_flags)
     else:
-        convert_image(src, dst, 'raw', True)
+        qemu_img.convert_image(src, dst, 'raw', True, sparse_size='0')
 
 
 def block_uuid(dev):
@@ -537,7 +497,7 @@ def get_image_mb(image_path, virtual_size=True):
     if not virtual_size:
         image_byte = os.path.getsize(image_path)
     else:
-        data = qemu_img_info(image_path)
+        data = qemu_img.image_info(image_path)
         image_byte = data.virtual_size
 
     # round up size to MB
@@ -592,7 +552,7 @@ def destroy_disk_metadata(dev, node_uuid):
     if dev_size < GPT_SIZE_SECTORS:
         dd_count = 'count=%s' % dev_size
     utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
-                  run_as_root=True, use_standard_locale=True)
+                  'oflag=direct', run_as_root=True, use_standard_locale=True)
 
     # Overwrite the Secondary GPT, do this only if there could be one
     if dev_size > GPT_SIZE_SECTORS:
@@ -600,7 +560,8 @@ def destroy_disk_metadata(dev, node_uuid):
         dd_seek = 'seek=%i' % gpt_backup
         dd_count = 'count=%s' % GPT_SIZE_SECTORS
         utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
-                      dd_seek, run_as_root=True, use_standard_locale=True)
+                      'oflag=direct', dd_seek, run_as_root=True,
+                      use_standard_locale=True)
 
     # Go ahead and let sgdisk run as well.
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
@@ -690,7 +651,7 @@ def udev_settle():
 def partprobe(device, attempts=None):
     """Probe partitions on the given device.
 
-    :param device: The block device containing paritions that is attempting
+    :param device: The block device containing partitions that is attempting
                    to be updated.
     :param attempts: Number of attempts to run partprobe, the default is read
                      from the configuration.
@@ -727,7 +688,7 @@ def trigger_device_rescan(device, attempts=None):
     When ramdisks are not in use, this also helps ensure that data has
     been safely flushed across the wire, such as on iscsi connections.
 
-    :param device: The block device containing paritions that is attempting
+    :param device: The block device containing partitions that is attempting
                    to be updated.
     :param attempts: Number of attempts to run partprobe, the default is read
                      from the configuration.
@@ -739,6 +700,7 @@ def trigger_device_rescan(device, attempts=None):
     # kernel.
     udev_settle()
     partprobe(device, attempts=attempts)
+    udev_settle()
     try:
         # Also verify that the partitioning is correct now.
         utils.execute('sgdisk', '-v', device, run_as_root=True)
